
unit unephjpl; (* for the JPL planet Moon Sun epehemeris *)

interface

uses
  unsagcot ; (* tvect3   const SunFm for example *)

const
  numberofplanets = 11 ;

 Const
   { for DE200 model only
     from Introduction aux Ephemerides Astronomiques  France }
   SystemOfMasses   : Array[1..NumberOfPlanets] of Extended =
                   (   6023600.000,         { Mercury IAU 1976}
                        408523.500,         { Venus IAU 1976 }
                        332946.0379476187,  { Earth only DE200 }
                       3098710.000,         { Mars IAU 1976 }
                          1047.350,         { Jupiter  DE200 }
                          3498.000,         { Saturn DE200 }
                         22960.000,         { Uranus DE200 }
                         19314.000,         { Neptune }
                     130000000.000,         { Pluto DE200 }
                      27068708.7489041655,  { the Moon only LE200 }
                             1.0  ) ;       { the Sun }

const
  massratio    = 0.012300034 ;
  coefmassratio= massratio/(1+massratio) ;

 Const
   NumCoefOrd : array[1..NumberOfPlanets] of integer =
            ( 2, 146, 182, 272, 302, 329, 353, 377, 395, 413, 701 );
   NumbofCoef : array[1..NumberOfPlanets] of integer =
            (12,  12,  15,  10,   9,   8,   8,   6,   6,  12,  15 );
   NumbofDay  : array[1..NumberOfPlanets] of integer =
            ( 8,  32,  16,  32,  32,  32,  32,  32,  32,   4,  32 );
   NumbofCoor : array[1..NumberOfPlanets] of integer =
            ( 3,   3,   3,   3,   3,   3,   3,   3,   3,   3,   3 );
   NMercury  = 1;
   NVenus    = 2;
   NEarth    = 3;
   NMars     = 4;
   NJupiter  = 5;
   NSaturn   = 6;
   NUranus   = 7;
   NNeptune  = 8;
   NPluto    = 9;
   NMoon     = 10;
   NSun      = 11;
   PlanetName    : array[1..NumberOfPlanets] of string[7] =
                   ( 'Mercury' , '  Venus' , '  Earth' , '   Mars' ,
                     'Jupiter' , ' Saturn' , ' Uranus' , 'Neptune' ,
                                 '  Pluto' , '   Moon' , '    Sun' );

 Type
   TypeCoef745     = Array [1..745] of double ;

 { to read ephemeris }
 Var
   CoeFile                      : File of TypeCoef745;
   CoefEph                      : TypeCoef745;
   CoeFileName                  : String;
   TinitEph,TlastEph            : extended ;
   NofRec,irecmax               : longint ;
   booopencoefile               : Boolean ;
   BooExistEph,BooExistPos      : Boolean ;

 Var
   { for each planet in AU**3/day**2 }
   FactorMuS  : Array [1..NumberOfPlanets] of Extended ;
   FactorMuZ  : Array [1..9] of Extended ; { from Mercury to Pluto }

 Procedure ToObtainFMuS ;

 Procedure ClcPosVel ( NPlanet  : byte ;
                       TinTDB   : extended ;
                       Var  Pos , Vel : tvect3 ) ; { type unsagcot }

 Procedure ToGetCoeFileName ; { all variables are external }

 Procedure ToReaDEphemeris ( TinTDB  : extended ) ;

 Procedure ClcEphEarthMoon ( TinTDB : Extended ;
                             Var  PosEarth , VelEarth : TypeDim3 ;
                             Var  PosMoon  , VelMoon  : TypeDim3 ) ;

procedure earths ( tb : extended ;  var  x , v : tvect3 ) ;

implementation

 { const SqrGauK from UnSagCot
   k=0.01720209895  SqrGauK = k^2 in unit
   (AU^3)*(day)^(-2)*(Sun mass)^(-1) const UnAllCon }
 procedure ForFactoM ; { var FactorMuPlanet from UnElmIni }
 var                   { const SystemOfMasses from UnElmIni }
   n  : Byte ;         // masses of the planets
   f  : Extended ;     // are in unit of the mass of the Sun
 begin
   f:=SqrGauK;       // for the Sun with mass=1 (AU^3)*(day)^(-2)
   for n:=1 to 8 do  // for each outer planet
     begin           // it is need to sum mass of the Sun
       f:=f+SqrGauK/SystemOfMasses[n]; // and masses
       FactorMuZ[n]:=f;     // of all inner planets plus our
     end;  // FactorMuZ  mu[i]=f*(m_0+m_1+m_2+...+m_(i-1)+m_i
   { unit ((Astronomical Unit)^3)/(Ephemeris Day)^2 }
   { but for the Pluto without Neptune }
   FactorMuZ[9]:=FactorMuZ[7]+SqrGauK/SystemOfMasses[9];
 end;      // unit ((Astronomical Unit)^3)/(Ephemeris Day)^2

 Procedure ToObtainFMuS ;
 Var
   nump  : Byte ;
 Begin
   { const unsagcot planetocentric gravitational constant }
   For nump:=1 To NumberOfPlanets Do   { in AU**3/day**2  }
     FactorMuS[nump]:=Sqr(GaussConst)/SystemOfMasses[nump];
   ForFactoM ; { gravitational constants with adding part }
 End;

 { ClcPosVel is a procedure
   to obtain the position of the Planet
      reffered to the barycentre from Ephemeris }

 Procedure ClcPosVel ( NPlanet  : byte ;
                       TinTDB   : extended ;
                       Var  Pos , Vel : tvect3 ) ;
 Var
   PolQuanT,PolQuanU : array[0..15] of extended;
   CoefQuant : array[1..3,0..15] of extended;
   Tn,Tk,Tnn,Tkk,t,r,w,rw,x,xw,xr,xv : extended;
   i,i1,i2,j,k,l,l0,m,n,np : integer;
 Begin
   ToReadEphemeris(TinTDB);
   If  NOT BooExistPos
     Then
       Exit ;
   k:=NumCoefOrd[NPlanet]; l:=NumbofCoef[NPlanet];
   l0:=l-1; m:=NumbofDay[NPlanet]; w:=m; rw:=2/w;
   t:=TinTDB; Tn:=CoefEph[1]; Tk:=Tn+32;
   r:=(t-Tn)/m; n:=Trunc(r-1.0e-10); np:=k+3*l*n-1;
   Tnn:=Tn+m*n; Tkk:=Tnn+m; x:=-1+(t-Tnn)*rw; xw:=2*x;
   for i:=1 to 3 do   for j:=0 to l0 do   begin
       np:=np+1; CoefQuant[i,j]:=CoefEph[np]; end;
   PolQuanT[0]:=1; PolQuanT[1]:=x; PolQuanU[0]:=1; PolQuanU[1]:=xw;
   for i:=2 to l do Begin  i1:=i-1; i2:=i-2;
       PolQuanT[i]:=xw*PolQuanT[i1]-PolQuanT[i2];
       PolQuanU[i]:=xw*PolQuanU[i1]-PolQuanU[i2]; End;
   for i:=1 to 3 do
     Begin
       xr:=CoefQuant[i,0]; xv:=0.0;
       for j:=1 to l0 do
         Begin
           xr:=xr+CoefQuant[i,j]*PolQuanT[j];
           xv:=xv+j*CoefQuant[i,j]*PolQuanU[j-1];
         End;
       Pos[i]:=xr/astrunit;     { position in AU }
       Vel[i]:=rw*xv/astrunit; { velosity in AU/day }
     End;
 End;

 { ToGetCoeFileName is a procedure
   to define name of the file for ephemeris }

 Procedure ToGetCoeFileName ; { all variables are external }
 Const
   numfile   = 1 ;
   namefile  : Array [1..numfile] of String =
             ( 'ephtheo.bin' ) ;
 Var
   num  : Byte ;
   Pos  : LongInt;
   ior  : Integer ;
   stn  : String ;
 Begin
   BooExistEph:=FALSE;
   BooExistPos:=FALSE;
   num:=0;
   Repeat
     num:=num+1;
     stn:=namefile[num];
     CoeFileName:=stn;
     Assign(CoeFile,CoeFileName);
     filemode:=0;
 {$I-}
     ReSet(CoeFile);
 {$I+}
     ior:=IOResult;
     filemode:=2;
     If  ior = 0
       Then
         BooExistEph:=TRUE ;
   Until  ( ( BooExistEph ) OR ( num = numfile ) ) ;
   If  NOT BooExistEph
     Then { no ephemeris file }
       Exit;
   { file is opened }
   Pos:=0; Seek(CoeFile,Pos);
   Read(CoeFile,CoefEph); TinitEph:=CoefEph[1];
   Pos:=FileSize(CoeFile)-1; Seek(CoeFile,Pos);
   Read(CoeFile,CoefEph); TlastEph:=CoefEph[1]+32;
   BooOpenCoeFile:=TRUE;
   NofRec:=-1;                   { global longint variable }
   irecmax:=FileSize(CoeFile)-1; { global longint variable }
 End;

 { to read file with the ephemeris data }

 Procedure ToReaDEphemeris ( TinTDB  : extended ) ;
 Var
   irec    : longint ;
   r       : extended;
 Begin
   BooExistPos:=FALSE; { previous }
   If  NOT BooOpenCoeFile
     Then
       ToGetCoeFileName;
   If  NOT BooExistEph
     Then { no ephemeris file }
       Exit;
   If  ( ( TinitEph <= TinTDB ) AND ( TinTDB <= TlastEph ) )
     Then
       Begin
         r:=(TinTDB-TinitEph)/32.e0;
         irec:=Trunc(r);
         If  irec > irecmax
           Then
             irec:=irecmax;
         If  irec <> NofRec
           Then
             Begin
               Seek(CoeFile,irec);
               Read(CoeFile,CoefEph);
               NofRec:=irec;
             End;
         BooExistPos:=TRUE; { there are ephemeris data }
       End;
  { Ephemeris has been read in massive  CoefEph(745) }
 End;

 { ClcEphEarthMoon is a procedure
   to obtain the position of the Earth and the Moon
      reffered to the barycenter of the Solar system from Ephemeris }

 Procedure ClcEphEarthMoon ( TinTDB : Extended ;
                             Var  PosEarth , VelEarth : TypeDim3 ;
                             Var  PosMoon  , VelMoon  : TypeDim3 ) ;
 Var
   i  : Byte ;
   posm,velm  : TypeDim3 ;
   PosBarEM,VelBarEM : TypeDim3 ;
 Begin
   ClcPosVel(NMoon,TinTDB,posm,velm); { reffered to the Earth }
   If  NOT BooExistPos
     Then { no ephemeris for this date TinTDB }
       Exit ;
   ClcPosVel(NEarth,TinTDB,PosBarEM,VelBarEM);
   For i:=1 To 3 Do
     Begin { reffered to the Solar system barycentre }
       PosEarth[i]:=PosBarEM[i]-CoefMassRatio*posm[i];
       VelEarth[i]:=VelBarEM[i]-CoefMassRatio*velm[i];
       PosMoon[i]:=PosEarth[i]+posm[i];
       VelMoon[i]:=VelEarth[i]+velm[i];
     End;
 End;

 (* earths in TDB moment tb is a procedure
    to obtain the position of the Earth reffered to the Sun *)

procedure earths ( tb  : extended ;  var  x , v  : tvect3 ) ;
var
  i  : byte ;
  psun,vsun,
  PosMoon,VelMoon,
  PosBarEM,VelBarEM : tvect3 ;
begin
  ClcPosVel(NSun,tb,psun,vsun);
  ClcPosVel(NMoon,tb,PosMoon,VelMoon);
  If  NOT BooExistPos
    Then { no ephemeris for this date TinTDB }
      Exit ;
  ClcPosVel(NEarth,tb,PosBarEM,VelBarEM);
  For i:=1 To 3 Do
    Begin
      x[i]:=PosBarEM[i]-CoefMassRatio*PosMoon[i]-psun[i];
      v[i]:=VelBarEM[i]-CoefMassRatio*VelMoon[i]-vsun[i];
    End;
end;

end.
