
 { for the numerical integration
   system of differential equations of the first order
   method presented by professor Everhart E. Celestial Mech., 1974, v.10 }

 UNIT UnEverGo ;

 INTERFACE

 Uses
   UnSagCot ; { for const type TVarSix }

 Var
   StepRada  : Array[1..8] of Extended ;
   DimCoefV  : Array[1..8] of Extended ;
   CoefEver  : Array[1..8] of TVarSix ;
   EverAlfa  : Array[1..8] of TVarSix ;
   MemAlpha  : Byte ;
   AlphaMom  : Array[1..2] of Array[1..NumberOfVar+1] of Extended ;

 var 
   maxvar,maxvau  : Byte ; { with value NumberOfVar from UnSagCot }

 Procedure EvorInit ;

 Procedure EvorCoefNullo ;

 Procedure VarFromPolinom ( DStep  : Extended ;
                            ZeroV  : TVarSix ;
                            Var
                              VectV  : TVarSix ) ;

 Procedure EverGratorFord ( StepFull  : Extended ;
                            Var
                              TBeg    : Extended ;
                            Var
                              ZeroV   : TVarSix ;
                            Var
                              TEnd    : Extended ;
                            Var
                              VectV   : TVarSix ) ;

 IMPLEMENTATION

 Uses
   UnRightS ; { for the procedure SecuRightHand }

 Var
   DifT  : Array[1..8] of Extended ;
   Coef  : Array[1..8] of Array[1..8] of Extended ;

 procedure EvorInit ; { initial value to some variables }
 begin
   StepRada[1]:=0.000000000000000000;
   StepRada[2]:=0.056262560526922147;
   StepRada[3]:=0.180240691736892365;
   StepRada[4]:=0.352624717113169637;
   StepRada[5]:=0.547153626330555383;
   StepRada[6]:=0.734210177215410532;
   StepRada[7]:=0.885320946839095768;
   StepRada[8]:=0.977520613561287501;
   DimCoefV[1]:=1.000000000000000000;
   DimCoefV[2]:=0.500000000000000000;
   DimCoefV[3]:=0.333333333333333333;
   DimCoefV[4]:=0.250000000000000000;
   DimCoefV[5]:=0.200000000000000000;
   DimCoefV[6]:=0.166666666666666667;
   DimCoefV[7]:=0.142857142857142857;
   DimCoefV[8]:=0.125000000000000000;
   maxvar:=NumberOfVar;   { UnForTyp global const }
   maxvau:=NumberOfVar+1; { UnForTyp global const }
 end;

 Procedure EvorCoefNullo ;  { initial moment nullo to start }
 Var
   i,j  : Byte ;
 Begin
   For i:=1 To 8 Do
     For j:=1 To maxvar Do
       Begin
         CoefEver[i][j]:=0.0;
         EverAlfa[i][j]:=0.0;
       End;
   MemAlpha:=0;
   For i:=1 To 2 Do
     For j:=1 To maxvau Do
       AlphaMom[i][j]:=0.0;
 End;

 { to obtain vector
      on the moment  t  from the coefficients of the polinoms
      constructed in integrator process presented by professor Everhart }


 Procedure VarFromPolinom ( DStep  : Extended ;
                            ZeroV  : TVarSix ;
                            Var
                              VectV  : TVarSix ) ;
 Var  { DStep in day of time from nullo point }
   i,j   : Byte ;
   CurV  : TVarSix ;
 Begin
   For j:=1 To maxvar Do
       CurV[j]:=DimCoefV[8]*CoefEver[8][j];
   For i:=7 DownTo 1 Do
     For j:=1 To maxvar Do
       CurV[j]:=DimCoefV[i]*CoefEver[i][j]+CurV[j]*DStep;
   For j:=1 To maxvar Do
     VectV[j]:=ZeroV[j]+CurV[j]*DStep; { for each variable }
 End;

 Procedure FromPrevious ( Var  TBeg   : Extended ;
                          Var  ZeroV  : TVarSix ;
                          Var  TEnd   : Extended ;
                          Var  VectV  : TVarSix ) ;
 Var
   i  : Byte ;
 Begin
   TBeg:=TEnd; { to change condition from previous step }
   For i:=1 To maxvar Do
     ZeroV[i]:=VectV[i]; { initial vector for each variable }
 End;

 Procedure CoeFromRecur ( StepFull  : Extended ) ;
 Var
   i,j  : Byte ;
 Begin
   DifT[1]:=0; { StepFull in day of time }
   For i:=2 To 8 Do
     DifT[i]:=StepRada[i]*StepFull;
   For i:=1 To 8 Do
     For j:=1 To 8 Do  Coef[i][j]:=0;
   { the simple coefficients from recurrence formula }
   For i:=1 To 8 Do  Coef[i][i]:=1;
   For i:=2 To 8 Do
     Coef[i][1]:=-DifT[i]*Coef[i-1][1];
   For i:=3 To 8 Do
     For j:=2 To i-1 Do
       Coef[i][j]:=Coef[i-1][j-1]-DifT[i]*Coef[i-1][j];
 End;

 Procedure CoeFromAlpha ( icur  : Byte );
 Var
   ia,j,k  : Byte ;
 Begin
   For j:=1 To maxvar Do
     For ia:=2 To icur Do   { to improve the value of coefficients }
       Begin
         CoefEver[ia][j]:=EverAlfa[ia][j];
         For k:=ia+1 To 8 Do
           CoefEver[ia][j]:=CoefEver[ia][j]
                           +Coef[k-1][ia-1]*EverAlfa[k][j];
       End;
 End;

 Procedure ToChangeAlpha ( TBeg  : Extended ) ;
 Var
   j  : Byte ;
   d  : Extended ;
 Begin
   If  MemAlpha < 2  Then  Exit ; { no action no alpha values in memory }
   d:=(TBeg-AlphaMom[2][maxvau])/(AlphaMom[2][maxvau]-AlphaMom[1][maxvau]);
   For j:=1 To maxvar Do { linear interpolation }
     EverAlfa[2][j]:=AlphaMom[2][j]+(AlphaMom[2][j]-AlphaMom[1][j])*d;
   CoeFromAlpha(2);  { to improve the value of the second coefficients }
 End;

 Procedure AlphaValToMem ( TBeg  : Extended ) ;
 Var
   j  : Byte ;
 Begin
   MemAlpha:=MemAlpha+1;
   If  MemAlpha = 3
     Then
       Begin
         MemAlpha:=2;
         For j:=1 To maxvar Do
           AlphaMom[1][j]:=AlphaMom[2][j];
         AlphaMom[1][maxvau]:=AlphaMom[2][maxvau];
       End;
   For j:=1 To maxvar Do
     AlphaMom[MemAlpha][j]:=EverAlfa[2][j];
   AlphaMom[MemAlpha][maxvau]:=TBeg;
 End;

 { to integrate the full function
      by the use of the method presented by professor Everhart }

 Procedure EverGratorFord ( StepFull  : Extended ;
                            Var
                              TBeg    : Extended ;
                            Var
                              ZeroV   : TVarSix ;
                            Var
                              TEnd    : Extended ;
                            Var
                              VectV   : TVarSix ) ;
 Var
   i,j,ia,it  : Byte ;
   tc,dst,dt  : Extended ;
   FunB,FunC  : TVarSix ;
   VCur,FCur  : TVarSix ;
 Begin

   FromPrevious(TBeg,ZeroV,TEnd,VectV);{ to new condition }

   If  Abs(StepFull) < 1.0e-12  Then  Exit; { no step needs }

   CoeFromRecur(StepFull); { StepFull in day of time }

   { value of the forces at the first point  t0 }
   SecuRightHand(TBeg,ZeroV,FCur); { from UnRightS TBeg in day from nullo }

   for j:=1 to maxvar do
     FunB[j]:=FCur[j]; { the right hand of equations }

   { initial values of the coefficients of the polinoms for forces }
   For j:=1 To maxvar Do
     CoefEver[1,j]:=FunB[j]; { the first coefficients }

   ToChangeAlpha(TBeg); { if there are two previous alpha values }

   it:=0;
 Repeat
   it:=it+1;
   For i:=2 To 8 Do
     Begin

       dst:=DifT[i];
       VarFromPolinom(dst,ZeroV,VCur);

       tc:=TBeg+DifT[i]; { step in day tc in day from initial moment }

       { value of the forces at the next point i }
       SecuRightHand(tc,VCur,FCur); { from UnRightS }

       for j:=1 to maxvar do
         FunC[j]:=FCur[j]; { the right hand of equations }

       dt:=1.0/DifT[i];
       For j:=1 To maxvar Do
         EverAlfa[i][j]:=(FunC[j]-FunB[j])*dt;

       { to get the next alpha value }
       For ia:=3 To i Do
         Begin
           dt:=1.0/(DifT[i]-DifT[ia-1]);
           For j:=1 To maxvar Do
             EverAlfa[i][j]:=(EverAlfa[i][j]-EverAlfa[ia-1][j])*dt;
         End;

       CoeFromAlpha(i);  { to improve the value of coefficients }

     End;
 Until it = 4 ;

   { coefficients of the polinoms have been calculated
     it is possible to obtain the vectors at the last point }
   TEnd:=TBeg+StepFull; { StepFull in day }
   VarFromPolinom(StepFull,ZeroV,VectV);

   AlphaValToMem(TBeg) ;

 End;

 END.
