
 { simple procedures to work with the Kepler elements }

 unit unforkep ;

 INTERFACE

 Uses
   unsagcot ; { for tvect3 or array [1..3] of extended }

 Function FromAxisToMot ( a  : Extended ) : Extended ;

 Function KeplerEquation ( ElemE , MeanAnom  : Extended ) : Extended ;

 Procedure FromKeplerToCart ( PosVar , AngVar  : TVect3 ;   { km second }
                              Var   Pos , Vel  : TVect3 ) ;

 Procedure fromkeplertus ( PosVar , AngVar  : TVect3 ;   { AU day }
                           Var   Pos , Vel  : TVect3 ) ;

 Procedure ForKeplerElem ( ta  : Extended ;  pa , ua  : TVect3 ;
                           tb  : Extended ;  Var  pb , ub  : TVect3 ) ;

 Procedure ToGetKeplerElements ( xc , vc  : tvect3 ;   { pos vel }
                                 var  ep  : tvect3 ;   { a e i }
                                 var  eu  : tvect3 ) ; { O o M }

 Procedure fromkeplerpos ( PosVar    : tvect3 ;
                           AngVar    : TVect3 ;
                           Var  pos  : TVect3 ) ;

 IMPLEMENTATION

 uses
   unforfun , { for simple function datan2 }
   unephjpl , { global arrays FactorMuS[..] FactorMuZ[..] }
   ungiamod ; { var PlanetFMain (AU^3/day^2) PlanetFM (km^2/s^2) }

 { for the satellite of the main planet
   MeanMotion   : mean motion  n  in radian per day
   semi-major axis in AU
   Sqr(GaussConst)  in AU**3/(day**2*SolarMass)
   from the formula
   n^2*a^3=k^2*(planet mass/SolarMass)
   where  k = 0.01720209895 - Gauss const
   or  n=sqrt(fm/a^3) fm=k^2*(planet mass/SolarMass) in AU^3/day^2 }

 Function FromAxisToMot ( a  : Extended ) : Extended ;
 Begin                                       { var from ungiamod }
   FromAxisToMot:=Sqrt(PlanetFMain/(a*a*a)); { in radian per day }
 End;

 { Kepler equation   M=E-e*sin(E)
   to solve with the use Newton iteration method
   from the book
   ...   ᬨ᪮ .
   .,㪠,1965.
     ࠭ 113:
   If  fi(x)=0  then iteration may be possible
   x_(n+1)=f(x_n)  where  f(x)=x-fi(x)/(d/dx(fi(x))) }

 Function KeplerEquation ( ElemE , MeanAnom  : Extended ) : Extended ;
 Var
   iter  : Byte ;
   EccAnom  : Extended ;
   cura,difa  : Extended ;
 Begin
   cura:=MeanAnom+ElemE*Sin(MeanAnom); { in radian }
   EccAnom:=cura;
   difa:=1.0;
   iter:=0;
   While  ( ( difa > 1.0e-15 ) AND ( iter < 241 ) )  Do
     Begin
       EccAnom:=cura-(cura-ElemE*Sin(cura)-MeanAnom)/(1-ElemE*Cos(cura));
       difa:=Abs(cura-EccAnom);
       cura:=EccAnom;
       iter:=iter+1;
     End;
   KeplerEquation:=EccAnom; { eccentric anomaly in radian }
 End;

 { to change from
   Kepler osculating elements to Descart position and velosity
   radgra unsagcot for angle from radian to degree
   PosVar 1,2,3 : a in AU, e,  Inclination in degree
   AngVar 1,2,3 : the ascending node in degree
                : the argument of perigei in degree
                : the mean anomaly un degree
   position Pos 1,2,3  : x,y,z in km 
   velosity Vel 1,2,3  : vx,vy,vz in km/s }

 Procedure FromKeplerToCart ( PosVar , AngVar  : TVect3 ;
                              Var   pos , vel  : TVect3 ) ;
 Var
   q,vr,vn  : Extended ;
   EccAnom  : Extended ;
   MeanAnom  : Extended ;
   TrueAnom  : Extended ;
   ArgLatitude  : Extended ;
   AscendingNode  : Extended ;
   r,xr,yr,zr,xv,yv,zv  : Extended ;
   ElemA,ElemE,CosInc,SinInc  : Extended ;
   CosE,SinE,CosV,SinV,CosU,SinU,CosNode,SinNode  : Extended ;
 Begin
   ElemA:=PosVar[1];
   ElemE:=PosVar[2];
   CosInc:=Cos(GraRad*PosVar[3]);
   SinInc:=Sin(GraRad*PosVar[3]);
   MeanAnom:=GraRad*AngVar[3]; { to radian }
   EccAnom:=KeplerEquation(ElemE,MeanAnom);
   SinE:=Sin(EccAnom);
   CosE:=Cos(EccAnom);
   SinV:=Sqrt(1-ElemE*ElemE)*SinE/(1-ElemE*CosE);
   CosV:=(CosE-ElemE)/(1-ElemE*CosE);
   TrueAnom:=EccAnom+ArcTan((SinV*CosE-CosV*SinE)/(CosV*CosE+SinV*SinE));
   ArgLatitude:=TrueAnom+GraRad*AngVar[2]; { in radian }
   SinU:=Sin(ArgLatitude);
   CosU:=Cos(ArgLatitude);
   AscendingNode:=GraRad*AngVar[1];
   SinNode:=Sin(AscendingNode);
   CosNode:=Cos(AscendingNode);
   r:=AstrUnit*ElemA*(1-ElemE*CosE); { from AU to km }
   xr:=+CosU*CosNode-SinU*SinNode*CosInc;
   yr:=+CosU*SinNode+SinU*CosNode*CosInc;
   zr:=+SinU*SinInc;
   xv:=-SinU*CosNode-CosU*SinNode*CosInc;
   yv:=-SinU*SinNode+CosU*CosNode*CosInc;
   zv:=+CosU*SinInc;
   q:=Sqrt(PlanetFM/(AstrUnit*ElemA*(1-Sqr(ElemE)))); { ungiamod km/s }
   vr:=q*ElemE*SinV;
   vn:=q*(1+ElemE*CosV);
   pos[1]:=r*xr; { in km }
   pos[2]:=r*yr;
   pos[3]:=r*zr;
   vel[1]:=vr*xr+vn*xv; { in km/s }
   vel[2]:=vr*yr+vn*yv;
   vel[3]:=vr*zr+vn*zv;
 End;

 { to change from
   Kepler osculating elements to Descart position and velosity
   radgra from unct for angle from radian to degree
   PosVar 1,2,3 : a in AU, e,  Inclination in degree
   AngVar 1,2,3 : the ascending node in degree
                : the argument of perigei in degree
                : the mean anomaly un degree
   position Pos 1,2,3  : x,y,z in AU
   velosity Vel 1,2,3  : vx,vy,vz in AU/day }

 Procedure fromkeplertus ( PosVar , AngVar  : TVect3 ;
                           Var   pos , vel  : TVect3 ) ;
 Var
   q,vr,vn  : Extended ;
   EccAnom  : Extended ;
   MeanAnom  : Extended ;
   TrueAnom  : Extended ;
   ArgLatitude  : Extended ;
   AscendingNode  : Extended ;
   r,xr,yr,zr,xv,yv,zv  : Extended ;
   ElemA,ElemE,CosInc,SinInc  : Extended ;
   CosE,SinE,CosV,SinV,CosU,SinU,CosNode,SinNode  : Extended ;
 Begin
   ElemA:=PosVar[1];
   ElemE:=PosVar[2];
   CosInc:=Cos(GraRad*PosVar[3]);
   SinInc:=Sin(GraRad*PosVar[3]);
   MeanAnom:=GraRad*AngVar[3]; { to radian }
   EccAnom:=KeplerEquation(ElemE,MeanAnom);
   SinE:=Sin(EccAnom);
   CosE:=Cos(EccAnom);
   SinV:=Sqrt(1-ElemE*ElemE)*SinE/(1-ElemE*CosE);
   CosV:=(CosE-ElemE)/(1-ElemE*CosE);
   TrueAnom:=EccAnom+ArcTan((SinV*CosE-CosV*SinE)/(CosV*CosE+SinV*SinE));
   ArgLatitude:=TrueAnom+GraRad*AngVar[2]; { in radian }
   SinU:=Sin(ArgLatitude);
   CosU:=Cos(ArgLatitude);
   AscendingNode:=GraRad*AngVar[1];
   SinNode:=Sin(AscendingNode);
   CosNode:=Cos(AscendingNode);
   r:=ElemA*(1-ElemE*CosE); { in AU }
   xr:=+CosU*CosNode-SinU*SinNode*CosInc;
   yr:=+CosU*SinNode+SinU*CosNode*CosInc;
   zr:=+SinU*SinInc;
   xv:=-SinU*CosNode-CosU*SinNode*CosInc;
   yv:=-SinU*SinNode+CosU*CosNode*CosInc;
   zv:=+CosU*SinInc;
   q:=Sqrt(PlanetFMain/(ElemA*(1-Sqr(ElemE)))); { ungiamod }
   vr:=q*ElemE*SinV;
   vn:=q*(1+ElemE*CosV);
   pos[1]:=r*xr; { in AU }
   pos[2]:=r*yr;
   pos[3]:=r*zr;
   vel[1]:=vr*xr+vn*xv; { in AU/day }
   vel[2]:=vr*yr+vn*yv;
   vel[3]:=vr*zr+vn*zv;
 End;

 Procedure ForKeplerElem ( ta  : Extended ;  pa , ua  : TVect3 ;
                           tb  : Extended ;  Var  pb , ub  : TVect3 ) ;
 Var
   n  : Extended ;
 Begin
   n:=radgra*FromAxisToMot(pa[1]); { mean motion in degree per day }
   pb:=pa; { the same elements }
   ub:=ua; { the same elements }
   ub[3]:=ua[3]+n*(tb-ta); { mean anomaly M in degree to new moment }
 End;

 { to change
   from Descart ecliptic position and velosity
   to kepler elements relative to planet in ecliptic plane}

 Procedure ToGetKeplerElements ( xc , vc  : tvect3 ;   { pos vel }
                                 var  ep  : tvect3 ;   { a e i }
                                 var  eu  : tvect3 ) ; { O o M }
 Var
   i                    : Byte;
   FactorMu,
   h,c,l,r,v2,q,
   c1,c2,c3,l1,l2,l3,
   a,e,p,rincl,
   node,argp,
   TrueAnom,
   EccAnom,MeanAnom,
   si,ci,su,cu,sn,cn,
   sp,cp,sv,cv,se,ce    : Extended ;
   Pos,Vel              : tvect3;
 Begin
   FactorMu:=PlanetFMain; { the main planet as central body ungiamod }
   For i:=1 To 3 Do
     Begin            { ecliptic }
       Pos[i]:=xc[i]; { position in AU }
       Vel[i]:=vc[i]; { velosity in AU/day }
     End;
   r:=Sqrt(Sqr(Pos[1])+Sqr(Pos[2])+Sqr(Pos[3])); { in AU }
   v2:=Sqr(Vel[1])+Sqr(Vel[2])+Sqr(Vel[3]); { in (AU/day)**2 }
   q:=FactorMu/r; h:=v2-2*q;
   c1:=Pos[2]*Vel[3]-Pos[3]*Vel[2];
   c2:=Pos[3]*Vel[1]-Pos[1]*Vel[3];
   c3:=Pos[1]*Vel[2]-Pos[2]*Vel[1];
   c:=Sqrt(c1*c1+c2*c2+c3*c3);
   l1:=-q*Pos[1]+Vel[2]*c3-Vel[3]*c2;
   l2:=-q*Pos[2]+Vel[3]*c1-Vel[1]*c3;
   l3:=-q*Pos[3]+Vel[1]*c2-Vel[2]*c1;
   l:=Sqrt(l1*l1+l2*l2+l3*l3);
   a:=-FactorMu/h; { in AU }
   e:=l/FactorMu;
   p:=c*c/FactorMu;
   ci:=c3/c; si:=Sqrt(Abs(1-ci*ci));
   If  si < 1.0e-11  Then
     Begin
       si:=1.0e-11; rincl:=0; { singularity inclination = 0 }
     End
                     Else  rincl:=datan2(si,ci); { i in radian }
   sn:=c1/(c*si); cn:=-c2/(c*si);
   node:=datan2(sn,cn); { in radian }
   sp:=l3/(l*si); cp:=(l1/l)*cn+(l2/l)*sn;
   argp:=datan2(sp,cp);
   q:=1/r; su:=q*Pos[3]/si; cu:=q*Pos[1]*cn+q*Pos[2]*sn;
   sv:=su*cp-cu*sp; cv:=cu*cp+su*sp; { u = v + omega }
   TrueAnom:=datan2(sv,cv);
   q:=1/(1+e*cv); se:=Sqrt(1-e*e)*sv*q; ce:=(cv+e)*q;
   EccAnom:=TrueAnom+ArcTan((se*cv-ce*sv)/(ce*cv+se*sv));
   MeanAnom:=EccAnom-e*se; { M = E - e * Sin(E) }
   ep[1]:=a; { semimajor axis AU }
   ep[2]:=e; { eccentricity }
   ep[3]:=RadGra*rincl; { inclination degree }
   eu[1]:=RadGra*node;  { ascending node degree }
   if  eu[1] > +180.0  then  eu[1]:=eu[1]-360.0;
   if  eu[1] < -180.0  then  eu[1]:=eu[1]+360.0;
   eu[2]:=RadGra*argp;  { argument of perihelion degree }
   if  eu[2] > +180.0  then  eu[2]:=eu[2]-360.0;
   if  eu[2] < -180.0  then  eu[2]:=eu[2]+360.0;
   eu[3]:=RadGra*MeanAnom; { mean anomaly degree }
   if  eu[3] > +180.0  then  eu[3]:=eu[3]-360.0;
   if  eu[3] < -180.0  then  eu[3]:=eu[3]+360.0;
 end;

 { to change from
   Kepler osculating elements to Descart position
   radgra unsagcot for angle from radian to degree
   PosVar 1,2,3 : a in AU, e,  Inclination in degree
   AngVar 1,2,3 : the ascending node in degree
                : the argument of perigei in degree
                : the mean anomaly un degree 
   output
   position Pos 1,2,3  : x,y,z in AU }

 Procedure fromkeplerpos ( PosVar    : tvect3 ;
                           AngVar    : TVect3 ;
                           Var  pos  : TVect3 ) ;
 Var
   EccAnom  : Extended ;
   MeanAnom  : Extended ;
   TrueAnom  : Extended ;
   ArgLatitude  : Extended ;
   AscendingNode  : Extended ;
   r,xr,yr,zr   : Extended ;
   ElemA,ElemE,CosInc,SinInc  : Extended ;
   CosE,SinE,CosV,SinV,CosU,SinU,CosNode,SinNode  : Extended ;
 Begin
   ElemA:=PosVar[1];
   ElemE:=PosVar[2];
   CosInc:=Cos(GraRad*PosVar[3]);
   SinInc:=Sin(GraRad*PosVar[3]);
   MeanAnom:=GraRad*AngVar[3]; { to radian }
   EccAnom:=KeplerEquation(ElemE,MeanAnom);
   SinE:=Sin(EccAnom);
   CosE:=Cos(EccAnom);
   SinV:=Sqrt(1-ElemE*ElemE)*SinE/(1-ElemE*CosE);
   CosV:=(CosE-ElemE)/(1-ElemE*CosE);
   TrueAnom:=EccAnom+ArcTan((SinV*CosE-CosV*SinE)/(CosV*CosE+SinV*SinE));
   ArgLatitude:=TrueAnom+GraRad*AngVar[2]; { in radian }
   SinU:=Sin(ArgLatitude);
   CosU:=Cos(ArgLatitude);
   AscendingNode:=GraRad*AngVar[1];
   SinNode:=Sin(AscendingNode);
   CosNode:=Cos(AscendingNode);
   r:=ElemA*(1-ElemE*CosE); { in AU }
   xr:=+CosU*CosNode-SinU*SinNode*CosInc;
   yr:=+CosU*SinNode+SinU*CosNode*CosInc;
   zr:=+SinU*SinInc;
   pos[1]:=r*xr; { in AU }
   pos[2]:=r*yr;
   pos[3]:=r*zr;
 End;

 END.
