
 UNIT UnForNut ; { constant }

 INTERFACE

 Uses
   UnSagCoT ; { for type TFundArg }


 function epsmean ( t  : extended ) : extended ;

 Procedure ClcFundArg ( Tjd  : Extended ;
                        Var  FundArg : TFundArg ) ;

 Procedure ClcNut ( Tjd : Extended ; { angles in radian }
                    Var  DeltaPsi , DeltaEps  : Extended ) ;

 IMPLEMENTATION

 (* epsmean mean angle of inclination ecliptic to equator *)

 function epsmean ( t  : extended ) : extended ;
 var
   dt  : extended ;
 begin
   dt:=(t-jd2000)/julianc;
   epsmean:=secrad*(84381.448
                    +dt*(-46.8150+dt*(-0.00059+dt*0.001813)));
 end;

 { ClcFundArg is a procedure
   to calculate Fundamental Arguments in radians
      on moment Tjd in Julian Date.
      FundArg[1..5] is massive of the values of arguments.
      global const - GraRad - for change from degrees to radians. }

 Procedure ClcFundArg ( Tjd  : Extended ;
                        Var  FundArg  : TFundArg ) ;
 Var
   r,dt,dt2,dt3 : Extended;
   i,l : Integer;
 Begin
   dt:=(Tjd-jd2000)/julianc; { UnAstCoT }
   dt2:=sqr(dt);
   dt3:=dt*dt2;
   FundArg[1]:=218.31643250e0+481267.8812772222e0*dt
                             -0.00161167e0*dt2+0.00000528e0*dt3;
   FundArg[2]:=134.96298139e0+477198.8673980556e0*dt
                             +0.00869722e0*dt2+0.00001778e0*dt3;
   FundArg[3]:=357.52772333e0+35999.05034e0*dt
                             -0.00016028e0*dt2-0.00000333e0*dt3;
   FundArg[4]:= 93.27191028e0+483202.0175380555e0*dt
                             -0.00368250e0*dt2+0.00000306e0*dt3;
   FundArg[5]:=297.85036306e0+445267.11148e0*dt
                             -0.00191417e0*dt2+0.00000528e0*dt3;
   for i:=1 to 5 do
     Begin
       r:=FundArg[i];
       l:=Trunc(r/360.e0);
       FundArg[i]:=GraRad*(r-360.e0*l); { UnAstCoT }
     End;
 End;

 { ClcNut is a procedure
   to calculate the parameters of nutation in longitude and obliquity
      and value of the mean obliquity in radians on the moment Tjd.
      the arguments and coefficients
          of nutation theory are contained in the NutArg and CoefNut .
      global parameter - SecRad - for change from arcsecs to radians . }

 Procedure ClcNut ( Tjd  : Extended;
                    Var  DeltaPsi , DeltaEps  : Extended ) ;
 Var
   i,k : Integer;
   r,dt   : Extended ;
   FundArg      : TFundArg ; { type from UnConTyp }
 Const
   { 1980 IAU Theory of Nutation  ( J. M. Wahr ) }
   NutArg   : Array [1..106,1..5] of ShortInt = { arguments }
            ( (   1 ,    0 ,    0 ,    0 ,    0 ) , 
              (   2 ,    0 ,    0 ,    0 ,    0 ) , 
              (   1 ,   -2 ,    0 ,    2 ,    0 ) , 
              (   0 ,    2 ,    0 ,   -2 ,    0 ) , 
              (   2 ,   -2 ,    0 ,    2 ,    0 ) , 
              (   0 ,    1 ,   -1 ,    0 ,   -1 ) , 
              (   1 ,    0 ,   -2 ,    2 ,   -2 ) ,
              (   1 ,    2 ,    0 ,   -2 ,    0 ) , 
              (   2 ,    0 ,    0 ,    2 ,   -2 ) , 
              (   0 ,    0 ,    1 ,    0 ,    0 ) , 
              (   2 ,    0 ,    1 ,    2 ,   -2 ) , 
              (   2 ,    0 ,   -1 ,    2 ,   -2 ) , 
              (   1 ,    0 ,    0 ,    2 ,   -2 ) , 
              (   0 ,    2 ,    0 ,    0 ,   -2 ) , 
              (   0 ,    0 ,    0 ,    2 ,   -2 ) , 
              (   0 ,    0 ,    2 ,    0 ,    0 ) , 
              (   1 ,    0 ,    1 ,    0 ,    0 ) ,
              (   2 ,    0 ,    2 ,    2 ,   -2 ) ,
              (   1 ,    0 ,   -1 ,    0 ,    0 ) , 
              (   1 ,   -2 ,    0 ,    0 ,    2 ) , 
              (   1 ,    0 ,   -1 ,    2 ,   -2 ) , 
              (   1 ,    2 ,    0 ,    0 ,   -2 ) , 
              (   1 ,    0 ,    1 ,    2 ,   -2 ) , 
              (   0 ,    1 ,    0 ,    0 ,   -1 ) , 
              (   0 ,    2 ,    1 ,    0 ,   -2 ) , 
              (   1 ,    0 ,    0 ,   -2 ,    2 ) ,
              (   0 ,    0 ,    1 ,   -2 ,    2 ) , 
              (   2 ,    0 ,    1 ,    0 ,    0 ) , 
              (   1 ,   -1 ,    0 ,    0 ,    1 ) , 
              (   0 ,    0 ,    1 ,    2 ,   -2 ) , 
              (   2 ,    0 ,    0 ,    2 ,    0 ) , 
              (   0 ,    1 ,    0 ,    0 ,    0 ) , 
              (   1 ,    0 ,    0 ,    2 ,    0 ) , 
              (   2 ,    1 ,    0 ,    2 ,    0 ) , 
              (   0 ,    1 ,    0 ,    0 ,   -2 ) , 
              (   2 ,   -1 ,    0 ,    2 ,    0 ) ,
              (   0 ,    0 ,    0 ,    0 ,    2 ) , 
              (   1 ,    1 ,    0 ,    0 ,    0 ) , 
              (   1 ,   -1 ,    0 ,    0 ,    0 ) , 
              (   2 ,   -1 ,    0 ,    2 ,    2 ) , 
              (   1 ,    1 ,    0 ,    2 ,    0 ) , 
              (   2 ,    0 ,    0 ,    2 ,    2 ) , 
              (   0 ,    2 ,    0 ,    0 ,    0 ) , 
              (   2 ,    1 ,    0 ,    2 ,   -2 ) , 
              (   2 ,    2 ,    0 ,    2 ,    0 ) ,
              (   0 ,    0 ,    0 ,    2 ,    0 ) , 
              (   1 ,   -1 ,    0 ,    2 ,    0 ) , 
              (   1 ,   -1 ,    0 ,    0 ,    2 ) , 
              (   1 ,    1 ,    0 ,    0 ,   -2 ) , 
              (   1 ,   -1 ,    0 ,    2 ,    2 ) , 
              (   0 ,    1 ,    1 ,    0 ,   -2 ) , 
              (   2 ,    0 ,    1 ,    2 ,    0 ) , 
              (   2 ,    0 ,   -1 ,    2 ,    0 ) , 
              (   2 ,    1 ,    0 ,    2 ,    2 ) ,
              (   0 ,    1 ,    0 ,    0 ,    2 ) , 
              (   2 ,    2 ,    0 ,    2 ,   -2 ) , 
              (   1 ,    0 ,    0 ,    0 ,    2 ) , 
              (   1 ,    0 ,    0 ,    2 ,    2 ) , 
              (   1 ,    1 ,    0 ,    2 ,   -2 ) , 
              (   1 ,    0 ,    0 ,    0 ,   -2 ) , 
              (   0 ,    1 ,   -1 ,    0 ,    0 ) , 
              (   1 ,    2 ,    0 ,    2 ,    0 ) , 
              (   0 ,    0 ,    1 ,    0 ,   -2 ) , 
              (   0 ,    1 ,    0 ,   -2 ,    0 ) ,
              (   0 ,    0 ,    0 ,    0 ,    1 ) , 
              (   0 ,    1 ,    1 ,    0 ,    0 ) , 
              (   0 ,    1 ,    0 ,    2 ,    0 ) , 
              (   2 ,    1 ,   -1 ,    2 ,    0 ) , 
              (   2 ,   -1 ,   -1 ,    2 ,    2 ) , 
              (   1 ,   -2 ,    0 ,    0 ,    0 ) , 
              (   2 ,    3 ,    0 ,    2 ,    0 ) , 
              (   2 ,    0 ,   -1 ,    2 ,    2 ) ,
              (   2 ,    1 ,    1 ,    2 ,    0 ) , 
              (   1 ,   -1 ,    0 ,    2 ,   -2 ) , 
              (   1 ,    2 ,    0 ,    0 ,    0 ) , 
              (   2 ,    1 ,    0 ,    0 ,    0 ) , 
              (   0 ,    3 ,    0 ,    0 ,    0 ) , 
              (   2 ,    0 ,    0 ,    2 ,    1 ) , 
              (   2 ,   -1 ,    0 ,    0 ,    0 ) , 
              (   0 ,    1 ,    0 ,    0 ,   -4 ) , 
              (   2 ,   -2 ,    0 ,    2 ,    2 ) , 
              (   2 ,   -1 ,    0 ,    2 ,    4 ) , 
              (   0 ,    2 ,    0 ,    0 ,   -4 ) ,
              (   2 ,    1 ,    1 ,    2 ,   -2 ) , 
              (   1 ,    1 ,    0 ,    2 ,    2 ) , 
              (   2 ,   -2 ,    0 ,    2 ,    4 ) , 
              (   2 ,   -1 ,    0 ,    4 ,    0 ) , 
              (   0 ,    1 ,   -1 ,    0 ,   -2 ) , 
              (   1 ,    2 ,    0 ,    2 ,   -2 ) , 
              (   2 ,    2 ,    0 ,    2 ,    2 ) ,
              (   1 ,    1 ,    0 ,    0 ,    2 ) , 
              (   2 ,    0 ,    0 ,    4 ,   -2 ) , 
              (   2 ,    3 ,    0 ,    2 ,   -2 ) , 
              (   0 ,    1 ,    0 ,    2 ,   -2 ) , 
              (   1 ,    0 ,    1 ,    2 ,    0 ) , 
              (   1 ,   -1 ,   -1 ,    0 ,    2 ) , 
              (   1 ,    0 ,    0 ,   -2 ,    0 ) , 
              (   2 ,    0 ,    0 ,    2 ,   -1 ) , 
              (   0 ,    0 ,    1 ,    0 ,    2 ) , 
              (   0 ,    1 ,    0 ,   -2 ,   -2 ) , 
              (   1 ,    0 ,   -1 ,    2 ,    0 ) , 
              (   1 ,    1 ,    1 ,    0 ,   -2 ) ,
              (   0 ,    1 ,    0 ,   -2 ,    2 ) , 
              (   0 ,    2 ,    0 ,    0 ,    2 ) , 
              (   2 ,    0 ,    0 ,    2 ,    4 ) , 
              (   0 ,    0 ,    1 ,    0 ,    1 ) ) ;
   { 1980 IAU Theory of Nutation  ( J. M. Wahr ) }
   CoefNut  : Array [1..106,1..4] of Single = { amplitudes }
    ( (   -17.199600 ,     -0.017420 ,      9.202500 ,      0.000890 ) , 
      (     0.206200 ,      0.000020 ,     -0.089500 ,      0.000050 ) , 
      (     0.004600 ,      0.000000 ,     -0.002400 ,      0.000000 ) , 
      (     0.001100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (    -0.000300 ,      0.000000 ,      0.000100 ,      0.000000 ) , 
      (    -0.000300 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (    -0.000200 ,      0.000000 ,      0.000100 ,      0.000000 ) , 
      (     0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (    -1.318700 ,     -0.000160 ,      0.573600 ,     -0.000310 ) , 
      (     0.142600 ,     -0.000340 ,      0.005400 ,     -0.000010 ) , 
      (    -0.051700 ,      0.000120 ,      0.022400 ,     -0.000060 ) , 
      (     0.021700 ,     -0.000050 ,     -0.009500 ,      0.000030 ) , 
      (     0.012900 ,      0.000010 ,     -0.007000 ,      0.000000 ) ,
      (     0.004800 ,      0.000000 ,      0.000100 ,      0.000000 ) , 
      (    -0.002200 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (     0.001700 ,     -0.000010 ,      0.000000 ,      0.000000 ) , 
      (    -0.001500 ,      0.000000 ,      0.000900 ,      0.000000 ) , 
      (    -0.001600 ,      0.000010 ,      0.000700 ,      0.000000 ) ,
      (    -0.001200 ,      0.000000 ,      0.000600 ,      0.000000 ) , 
      (    -0.000600 ,      0.000000 ,      0.000300 ,      0.000000 ) , 
      (    -0.000500 ,      0.000000 ,      0.000300 ,      0.000000 ) , 
      (     0.000400 ,      0.000000 ,     -0.000200 ,      0.000000 ) , 
      (     0.000400 ,      0.000000 ,     -0.000200 ,      0.000000 ) , 
      (    -0.000400 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (     0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (     0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (    -0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (     0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (     0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (    -0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (    -0.227400 ,     -0.000020 ,      0.097700 ,     -0.000050 ) , 
      (     0.071200 ,      0.000010 ,     -0.000700 ,      0.000000 ) ,
      (    -0.038600 ,     -0.000040 ,      0.020000 ,      0.000000 ) , 
      (    -0.030100 ,      0.000000 ,      0.012900 ,     -0.000010 ) , 
      (    -0.015800 ,      0.000000 ,     -0.000100 ,      0.000000 ) , 
      (     0.012300 ,      0.000000 ,     -0.005300 ,      0.000000 ) ,
      (     0.006300 ,      0.000000 ,     -0.000200 ,      0.000000 ) , 
      (     0.006300 ,      0.000010 ,     -0.003300 ,      0.000000 ) , 
      (    -0.005800 ,     -0.000010 ,      0.003200 ,      0.000000 ) , 
      (    -0.005900 ,      0.000000 ,      0.002600 ,      0.000000 ) , 
      (    -0.005100 ,      0.000000 ,      0.002700 ,      0.000000 ) , 
      (    -0.003800 ,      0.000000 ,      0.001600 ,      0.000000 ) , 
      (     0.002900 ,      0.000000 ,     -0.000100 ,      0.000000 ) , 
      (     0.002900 ,      0.000000 ,     -0.001200 ,      0.000000 ) , 
      (    -0.003100 ,      0.000000 ,      0.001300 ,      0.000000 ) , 
      (     0.002600 ,      0.000000 ,     -0.000100 ,      0.000000 ) , 
      (     0.002100 ,      0.000000 ,     -0.001000 ,      0.000000 ) , 
      (     0.001600 ,      0.000000 ,     -0.000800 ,      0.000000 ) , 
      (    -0.001300 ,      0.000000 ,      0.000700 ,      0.000000 ) , 
      (    -0.001000 ,      0.000000 ,      0.000500 ,      0.000000 ) , 
      (    -0.000700 ,      0.000000 ,      0.000000 ,      0.000000 ) ,
      (     0.000700 ,      0.000000 ,     -0.000300 ,      0.000000 ) , 
      (    -0.000700 ,      0.000000 ,      0.000300 ,      0.000000 ) , 
      (    -0.000800 ,      0.000000 ,      0.000300 ,      0.000000 ) ,
      (     0.000600 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (     0.000600 ,      0.000000 ,     -0.000300 ,      0.000000 ) , 
      (    -0.000600 ,      0.000000 ,      0.000300 ,      0.000000 ) , 
      (    -0.000700 ,      0.000000 ,      0.000300 ,      0.000000 ) ,
      (     0.000600 ,      0.000000 ,     -0.000300 ,      0.000000 ) , 
      (    -0.000500 ,      0.000000 ,      0.000300 ,      0.000000 ) , 
      (     0.000500 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (    -0.000500 ,      0.000000 ,      0.000300 ,      0.000000 ) , 
      (    -0.000400 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (     0.000400 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (    -0.000400 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (    -0.000300 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (     0.000300 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (    -0.000300 ,      0.000000 ,      0.000100 ,      0.000000 ) , 
      (    -0.000300 ,      0.000000 ,      0.000100 ,      0.000000 ) , 
      (    -0.000200 ,      0.000000 ,      0.000100 ,      0.000000 ) ,
      (    -0.000300 ,      0.000000 ,      0.000100 ,      0.000000 ) , 
      (    -0.000300 ,      0.000000 ,      0.000100 ,      0.000000 ) ,
      (     0.000200 ,      0.000000 ,     -0.000100 ,      0.000000 ) , 
      (    -0.000200 ,      0.000000 ,      0.000100 ,      0.000000 ) , 
      (     0.000200 ,      0.000000 ,     -0.000100 ,      0.000000 ) , 
      (    -0.000200 ,      0.000000 ,      0.000100 ,      0.000000 ) , 
      (     0.000200 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (     0.000200 ,      0.000000 ,     -0.000100 ,      0.000000 ) ,
      (     0.000100 ,      0.000000 ,     -0.000100 ,      0.000000 ) , 
      (    -0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (     0.000100 ,      0.000000 ,     -0.000100 ,      0.000000 ) , 
      (    -0.000200 ,      0.000000 ,      0.000100 ,      0.000000 ) , 
      (    -0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (     0.000100 ,      0.000000 ,     -0.000100 ,      0.000000 ) , 
      (    -0.000100 ,      0.000000 ,      0.000100 ,      0.000000 ) , 
      (    -0.000100 ,      0.000000 ,      0.000100 ,      0.000000 ) , 
      (     0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (     0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (     0.000100 ,      0.000000 ,     -0.000100 ,      0.000000 ) ,
      (    -0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) ,
      (    -0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (     0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) ,
      (     0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (    -0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (     0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (     0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (    -0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) ,
      (    -0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) ,
      (    -0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (    -0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (    -0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) , 
      (    -0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) ,
      (    -0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) ,
      (     0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) ,
      (    -0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) ,
      (     0.000100 ,      0.000000 ,      0.000000 ,      0.000000 ) ) ;
 Begin
   ClcFundArg(Tjd,FundArg);
   dt:=(Tjd-jd2000)/julianc;
   DeltaPsi:=0.e0;
   DeltaEps:=0.e0;
   for k:=1 to 106 do
     Begin
       r:=NutArg[k,1]*(FundArg[1]-FundArg[4]);
       for i:=2 to 5 do  r:=r+NutArg[k,i]*FundArg[i];
       DeltaPsi:=DeltaPsi+(CoefNut[k,1]
                       +dt*CoefNut[k,2])*Sin(r);
       DeltaEps:=DeltaEps+(CoefNut[k,3]
                       +dt*CoefNut[k,4])*Cos(r);
     End;
   DeltaPsi:=SecRad*DeltaPsi; { in radian }
   DeltaEps:=SecRad*DeltaEps; { UnAstCoT }
 End;

 END.
