unit UnSagElm ; { to change from usual and canonical elements }
interface
uses 
  unsagcot ; { types tsagelm tsagano tvarsix }
{ from osculating usual Kepler to canonical Delone elements }
procedure togetoscano ( el  : tsagelm ; var  xo  : tsagano ) ; { unsagcot }
{ to change from Delone to Kepler elements }
procedure togecanoscu ( xo  : tsagano ; var  el  : tsagelm ) ; { unsagcot }
implementation
uses
  ungiamod , { global variable PlanetFMain }
  unforfun , { function ATanDegree }
  unforprt ; { procedure ToControlBadCasePrt }
{ to change from Kepler to Delone elements }
procedure togetoscano ( el  : tsagelm ; var  xo  : tsagano ) ; { unsagcot }
begin
  xo.t:=el.epo; { epoch of the elements in julian day to record }
  { PlanetFMain plagrav constant in AU^3/day^2 global ungiamod }
  xo.e[1]:=sqrt(PlanetFMain*el.ela);    { element L=sqrt(fm*a) in AU^2/day }
  xo.e[2]:=xo.e[1]*sqrt(1.0-sqr(el.ele)); { G=sqrt(fm*a*(1-e^2)) in AU^2/day }
  xo.e[3]:=xo.e[2]*cos(grarad*el.eli);   { H=G*cos(i) in AU^2/day unsagcot }
  xo.e[4]:=grarad*el.elm;        { element l mean anomal in radian }
  xo.e[5]:=grarad*(el.elp+90.0); { element g=\omega+Pi/2 in radian }
  xo.e[6]:=grarad*(el.eln-90.0); { element h=\Omega-Pi/2 in radian }
  xo.n:=xo.e[1]/sqr(el.ela);     { mean motion in radian per day n=L/a^2 }
end;
{ to change from Delone to Kepler elements }
procedure togecanoscu ( xo  : tsagano ; var  el  : tsagelm ) ; { unsagcot }
var
  s,c  : extended ; { for sinus cosinus of inclination }
begin
  ToControlBadCasePrt(xo.t,xo.e); { unforprt may be L < G or G < H }
  el.epo:=xo.t; { epoch of the elements in julian day }
  el.ela:=sqr(xo.e[1])/PlanetFMain; { element a semimajor axis in AU L^2/fm }
                                          { e^2=1-G^2/L^2 }
  el.ele:=sqrt(1.0-sqr(xo.e[2]/xo.e[1])); { element e eccentricity }
  c:=xo.e[3]/xo.e[2]; { cosinus inclination = H/G may be plus or minus }
  s:=sqrt(1.0-sqr(c));     { sinus of inclination plus always }
  el.eli:=ATanDegree(s,c); { angle of inclination in degree 0 < i < 180 }
  el.eln:=radgra*xo.e[6]+90.0; { the ascending node in degree \Omega }
  el.elp:=radgra*xo.e[5]-90.0; { the argument of pericenter \omega }
  el.elm:=radgra*xo.e[4];      { the mean anomsly M in degree }
  while  el.eln > +180.0  do  el.eln:=el.eln-360.0; { to control \Omega max }
  while  el.eln < -180.0  do  el.eln:=el.eln+360.0; { to control \Omega min }
  while  el.elp > +180.0  do  el.elp:=el.elp-360.0; { to control \omega max }
  while  el.elp < -180.0  do  el.elp:=el.elp+360.0; { to control \omega min }
end;
end.
