
{ to prognoz with mean hamiltonian
  by using numerical integration method }

(*
type
  tsagelm  = record { to identify the current satellite }
               mig   : byte ;     { number of the main planet }
               snm   : string[16] ; { may be the number of the satellite }
               stn   : string[16] ; { may be the name }
               epo   : double ;   { elements epoch in julian day }
               ela   : double ;   { semimajor axis in astronomical unit }
               ele   : double ;   { eccentricity }
               eli   : double ;   { inclination in degree }
               eln   : double ;   { longitude of ascension node in degree }
               elp   : double ;   { argument of perihelion in degree }
               elm   : double ;   { mean anomaly in degree }
            end;
type
  tsagano  = record { canonical elements }
               t  : extended ; { epoch or current moment }
               e  : tvarsix ;  { canonical elements L G H l g h }
               n  : extended ; { mean motion in radian per day }
             end;
*)

unit UnSagSec ;

interface

uses
  UnSagCoT ; { for type tastel record for elements }

{ to prognoz by integration the secular orbit }

procedure SatGiaSec ( hd  : Extended ;  { step for integration }
                      hp  : extended ;  { step to print result }
                      ta  : Extended ;  { start  moment }
                      tb  : Extended ;  { finish moment }
                      el  : tsagelm ) ;  { UnSagCoT type elements }

implementation

uses
  untypvar , { global plaman byte for main planet }
  unsagosc , { procedure oscutomeone }
  unsecosc , { procedures frocusec ToScreenOuresult frosecos }
  unevergo , { for EverGrator procedures EvorInit EvorCoefNullo }
  unforprt , { protocol text  file fprt }
  unforstr ; { function FloatToSingle to convert numbers }

{ from planetocentric
  equator descart to geoecliptic kepler elements }

procedure ToPrintOuresult ( e  : tsagelm ); { for elements } 
begin
  with  e  do
    writeln(fprt,epo:10:2,  { epoch of the elements }
                 ela:9:5,  { a semimajor axis AU }
                 ele:9:6,  { e eccentricity }
                 eli:9:3,  { i inclination in degree }
                 eln:9:3,  { O the ascending node degree }
                 elp:9:3,  { o the argument of pericentre degree }
                 parall[9]/parall[10]:8:4,   { a/a' }
                 parall[13]/parall[14]:8:4,  { n/n' }
                 parall[14]/parall[13]:8:4); { n'/n }
end;

{ to prognoz by integration the secular orbit }

procedure SatGiaSec ( hd  : Extended ;  { step for integration }
                      hp  : extended ;  { step to print result }
                      ta  : Extended ;  { start  moment }
                      tb  : Extended ;  { finish moment }
                      el  : tsagelm ) ;  { UnSagCoT type elements }
var
  tc  : extended ; { current julian date }
  tp  : extended ; { julian day to print  result with step equal hp }
  ts  : extended ; { julian day to screen result with step 100 years }
  dt  : extended ; { step in day to integrate }
  xp  : tsagano ;  { secular canonical elements }
  xs  : tsagano ;  { current secular canonical record with t }
  eo  : tsagelm ;  { current osculat kepler elements }
  es  : tsagelm ;  { current secular kepler elements }
  za  : extended ; { small interval for integration start }
  zb  : extended ; { small interval for integration finish }
  zc  : extended ; { moment within small interval }
  ra  : TVarSix ;  { small interval for integration start  elements }
  rb  : TVarSix ;  { small interval for integration finish elements }
  rc  : TVarSix ;  { small interval elements for moment zc }

begin

  plaman:=el.mig; { global from untypvar of byte }

  frocusec(el,xp,es); { unsecosc from osculating to secular }

  dt:=hd;     { this is a value of step in day to integrate }
              { in tc is epoch of the elements as start moment }
  tc:=xp.t;   { this value may be greater than start moment ta }
  if  tc < ta  then  { but so may be less than start moment }
    begin            { no action }
      writeln('start moment is greater than elements epoch',
               ta:12:2,' > ',tc:12:2);
      Exit ;         { to go from the program }
    end;
  { initial condition to integrate to start moment ta }
  za:=xp.t; { initial moment }
  zb:=za;   { initial moment }
  ra:=xp.e; { initial elements secular canonical }
  rb:=ra;   { initial elements secular canonical }

  EvorInit ;       { UnEverGo some action to start EverGratorFord }
  EvorCoefNullo ;  { UnEverGo some action to start EverGratorFord }
  if  za > ta             { epoch of the elements }
    then                  { may be greater than start moment }
      while  ( (zb-ta) > 0.1 )  do  { backward to start moment }
        begin                       { but step may be too large }
          if  ( zb < (ta+dt) )  then  dt:=zb-ta; { to change step }
          EverGratorFord(-dt,za,ra,zb,rb); { UnEverGo one step }
        end;

  dt:=hd;     { this is a value of step in day to integrate }
              { in tc is epoch of the elements as start moment }
  tc:=zb;     { our start epoch for the elements }
  tp:=tc;     { julian day to print  results with step equal hp }
  ts:=tc;     { julian day to screen results with step 100 years }

  { start condition to integrate forward from our moment }
  za:=zb; { moment in julian day forward }
  ra:=rb; { start elements to integrate }

  xs.t:=zb;   { start moment of time to record }
  xs.e:=rb;   { start canonical elements to record }
  frosecos(xs,es,eo);   { unsecosc from secular to osculating }
  ToScreenOuresult(eo); { start ocsulating elements }
  ToPrintOuresult(eo);  { start ocsulating elements }

  EvorInit ;       { UnEverGo some action to start EverGratorFord }
  EvorCoefNullo ;  { UnEverGo some action to start EverGratorFord }

  while  tp < tb  do        { from start to finish moments }
    begin                   { step by step and analyse }

      tp:=tp+hp;            { one step to print result }
      while  tp < za  do    { current moment to interval }
        tp:=tp+hp;          { to the next moment }
                                        { moment to print to }
      while  ( zb < (tp-1.0e-5) )  do   { interval of integration }
        EverGratorFord(dt,za,ra,zb,rb); { UnEverGo one step }
                                        { za <= tp <= zb now }
      tc:=zb;   { the next moment to screen result }

      zc:=tp-za;                { step within current interval }
      VarFromPolinom(zc,ra,rc); { unevergo procedure to interpolate }

      xs.t:=tp; { current moment of time to record }
      xs.e:=rc; { current canonical elements to record }
      frosecos(xs,es,eo); { unsecosc from secular to osculating }
      if  ( (tc-ts) > 36525.0 )  then
        begin
          ToScreenOuresult(eo); { current ocsulating elements }
          ts:=tc; { the next current moment to screen }
        end;
      ToPrintOuresult(eo); { current ocsulating elements }
    end;

end;

end.
