unit UnTypVar ; { for different types for some case }
interface
const
  maxarr = 34361 ;  // dimension of arrays
  numsel = 14 ;     // selected elements
  maxarg = 8 ;      // number of arguments v E om Om  v' E' om' Om'
  maxder = 1 ;      // amplitude
type
  TAmpDer  = array[1..maxder] of Extended ; { amplitude }
  TDimSel  = array[1..numsel] of ShortInt ; { for power of variables }
  TDimArg  = array[1..maxarg] of ShortInt ; { for arguments }
  { elementary term has a form
     A * a^i_0 * n^i_5 * r^i_6
       * (sqrt(1/(1-e^2)))^i_1*e^i_2*(sin i)^i_3*(cos i)^i_4
       * a'^j_0 * n'^j_5 * r'^j_6
       * (sqrt(1/(1-e'^2)))^j_1 * e'^j_2*(sin i')^j_3 * (1-cos i')^j_4
       * cos(k_1*v~+k_2*E+k_3*g+k_4*h+l_1*v'~+l_2*E'+l_3*g'+l_4*h') }
  TRowVar  = record
               amp  : TAmpDer ; { amplitude }
               mec  : TDimSel ; { power of variables }
               mar  : TDimArg ; { argument }
               fcs  : Char ;    { function sinus or cosinus }
             end;
type
  { for the array }
  TRowDim  = array[1..maxarr] of TRowVar ;
type
  { for the pointer to array }
  TRowPtr  = ^TRowDim ;
type
  { for the partial derivations with canonical elements L G H }
  TAmpDif  = array[1..3] of extended ;
  TMecDif  = array[1..3] of TDimSel ;
const
  maxsel   : TDimSel  =   // maximum power
           ( 28 ,   // sqrt(1/(1-e^2))   1
             12 ,   // e                 2
             12 ,   // s=sin(i)          3
             12 ,   // c=cos(i)          4
             28 ,   // sqrt(1/(1-e'^2))  5
              4 ,   // e'                6
              4 ,   // s'=sin(i')        7
              2 ,   // 1-c'=1-cos(i')    8
             16 ,   // a                 9
             16 ,   // a'               10
             16 ,   // r                11
             16 ,   // r'               12
              5 ,   // n                13
              5 ) ; // n'               14
var
  maxsum  : byte ;    { maximum sum of powers for giant planets }
  numsum  : byte ;    { value maximum sum of powers }
  numpol  : byte ;    { maximum order for Legandre polinom }
  mecone  : Word ;    { number of terms in rowone array }
  mectwo  : Word ;    { number of terms in rowtwo array }
  mectre  : Word ;    { number of terms in rowtre array }
  mecfur  : Word ;    { number of terms in rowfur array }
  mecfiv  : Word ;    { number of terms in rowfiv array }
  mectmp  : Word ;    { number of terms in tempor array }
  meccoh  : word ;    { number of terms in cos(H) array }
  recnul  : TRowVar ; { for nullo record type from UnTypVar }
  rowone  : TRowPtr ; { to the pointer for array }
  rowtwo  : TRowPtr ; { to the pointer for array }
  rowtre  : TRowPtr ; { to the pointer for array }
  rowfur  : TRowPtr ; { to the pointer for array }
  rowfiv  : TRowPtr ; { to the pointer for array }
  rowtmp  : TRowPtr ; { pointer for tempor array }
  rowcoh  : TRowPtr ; { simple row for cos(H) }
var
  { for the partial derivations with canonical elements L G H }
  amplia  : TAmpDif ; { amplitude for a }
  amplip  : TAmpDif ; { amplitude for p }
  amplie  : TAmpDif ; { amplitude for e }
  amplis  : TAmpDif ; { amplitude for s }
  amplic  : TAmpDif ; { amplitude for c }
  amplin  : TAmpDif ; { amplitude for n }
  medisa  : TMecDif ; { partial derivations of semimajor axis }
  medipa  : TMecDif ; { partial derivations of (1/sqrt(1-e^2) }
  medecc  : TMecDif ; { partial derivations of e with L G H }
  mesini  : TMecDif ; { partial derivations of s=sin(i) }
  mecosi  : TMecDif ; { partial derivations of c=cos(i) }
  memoti  : TMecDif ; { partial derivations of n mean motion }
var       { numerical values for different powers of our p }
  parall  : array[1..numsel] of extended ; { num.val. of our parameters }
  powall  : array[1..numsel] of array[-9..+9] of extended ; { to power }
  argall  : array[1..maxarg] of extended ; { num.val. v E g h v' E' g' h' }
var
  plaman  : byte ;    { number of the main planet }
procedure GetRowMem ;
procedure FreeRowMem ;
implementation
procedure GetRowMem ;  { array to heap by pointer }
begin
  GetMem(rowone,SizeOf(TRowDim));
  GetMem(rowtwo,SizeOf(TRowDim));
  GetMem(rowtre,SizeOf(TRowDim));
  GetMem(rowfur,SizeOf(TRowDim));
  GetMem(rowfiv,SizeOf(TRowDim));
  GetMem(rowtmp,SizeOf(TRowDim));
  GetMem(rowcoh,SizeOf(TRowDim));
end;
procedure FreeRowMem ; { array to heap by pointer }
begin
  FreeMem(rowcoh,SizeOf(TRowDim));
  FreeMem(rowtmp,SizeOf(TRowDim));
  FreeMem(rowfiv,SizeOf(TRowDim));
  FreeMem(rowfur,SizeOf(TRowDim));
  FreeMem(rowtre,SizeOf(TRowDim));
  FreeMem(rowtwo,SizeOf(TRowDim));
  FreeMem(rowone,SizeOf(TRowDim));
end;
end.
